using static BlueWest.EfMethods.Sources.Constants;


namespace BlueWest.EfMethods.Sources
{
    internal static class EfAddMethodsAttributeSource
    {
        internal const string AttributeName = "EfAddMethods";
        internal const string AttributeClassName = AttributeName + "Attribute";
        internal const string FullyQualifiedName = RootNamespace + "." + AttributeClassName;
        
        internal static SourceCode Generate(SourceGenerationOptions options)
        {
            using var builder = new SourceBuilder()
                .WriteLine(GeneratedFilesHeader)
                .WriteLine("using System;")
                .WriteLine()
                .WriteLine($"namespace {RootNamespace}")
                .WriteOpeningBracket();

            if (options.GenerateXmlDocument)
            {
                builder
                    .WriteLine("/// <summary>")
                    .WriteLine("/// Generate Add methods for interacting with the entity")
                    .WriteLine("/// </summary>");
            }

            builder
                .WriteLine("[AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]")
                .WriteLine($"public sealed class {AttributeName}Attribute : Attribute")
                .WriteOpeningBracket();
            
            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Attribute for generating a method for creating a entity of the target property.")
                .WriteLine("/// <param name=\"createType\">A typeof with the Create projection type.</param>")
                .WriteLine("/// <param name=\"returnType\">The projection with the return data.</param>")
                .WriteLine("/// </summary>");

            builder
                .WriteLine($"public {AttributeName}Attribute(Type createType = null, Type returnType = null){"{}"}")
                .WriteLine();

            builder
                .WriteClosingBracket() // class
                .WriteClosingBracket(); // namespace

            return new(builder.ToString(), $"{AttributeName}Attribute.g.cs");
        }
    }
}