using BlueWest.EfMethods.Sources;
using static BlueWest.EfMethods.Sources.Constants;

namespace BlueWest.EfMethods.EfMethods.AttributeSources
{
    public static class EfAddToListAttributeSource
    {
        internal const string AttributeName = "EfAddToList";
        internal const string AttributeClassName = AttributeName + "Attribute";
        internal const string FullyQualifiedName = RootNamespace + "." + AttributeClassName;

        internal static SourceCode Generate(SourceGenerationOptions options)
        {
            using var builder = new SourceBuilder()
                .WriteLine(GeneratedFilesHeader)
                .WriteLine("using System;")
                .WriteLine()
                .WriteLine($"namespace {RootNamespace}")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Attribute for generating a function to get many entities.")
                .WriteLine("/// </summary>");
            

            builder
                .WriteLine("[AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]")
                .WriteLine($"public sealed class {AttributeName}Attribute : Attribute")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Attribute for generating a function to get many entities.")
                .WriteLine("/// <param name=\"listPropertyNameof\">The nameof property of the list.</param>")
                .WriteLine("/// <param name=\"createType\">The projection with the creation data.</param>")
                .WriteLine("/// <param name=\"returnType\">The projection type with the return data.</param>")
                .WriteLine("/// <param name=\"primaryKeyNameof\">Nameof with look up key of the entity containing the list.</param>")
                .WriteLine("/// </summary>");

            builder
                .WriteLine($"public {AttributeName}Attribute(string listPropertyNameof, Type createType, Type returnType, string primaryKeyNameof){"{}"}")
                .WriteLine();

            builder
                .WriteClosingBracket() // class
                .WriteClosingBracket(); // namespace

            return new(builder.ToString(), $"{AttributeName}Attribute.g.cs");
        }
    }
}