using BlueWest.EfMethods;
using BlueWest.EfMethods.Sources;
using static BlueWest.EfMethods.Sources.Constants;

namespace BlueWest.Sources
{
    public static class EfGetListAttributeSource
    {
        internal const string AttributeName = "EfGetList";
        internal const string AttributeClassName = AttributeName + "Attribute";
        internal const string FullyQualifiedName = RootNamespace + "." + AttributeClassName;

        internal static SourceCode Generate(SourceGenerationOptions options)
        {
            using var builder = new SourceBuilder()
                .WriteLine(GeneratedFilesHeader)
                .WriteLine("using System;")
                .WriteLine()
                .WriteLine($"namespace {RootNamespace}")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Attribute for generating a function to get many entities.")
                .WriteLine("/// </summary>");
            

            builder
                .WriteLine("[AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]")
                .WriteLine($"public sealed class {AttributeName}Attribute : Attribute")
                .WriteOpeningBracket();

                        
            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Gets records of a list.")
                .WriteLine("/// <param name=\"listPropertyNameof\">A nameof with the containing list.</param>")
                .WriteLine("/// <param name=\"returnType\">The projection with the return data.</param>")
                .WriteLine("/// <param name=\"primaryKeyNameof\">The lookup key of the main entity.</param>")
                .WriteLine("/// </summary>");
            builder
                .WriteLine($"public {AttributeName}Attribute(string listPropertyNameof, Type returnType, string primaryKeyNameof){"{}"}")
                .WriteLine();

            builder
                .WriteClosingBracket() // class
                .WriteClosingBracket(); // namespace

            return new(builder.ToString(), $"{AttributeName}Attribute.g.cs");
        }
    }
}