﻿using System;
using System.Collections.Generic;
using System.Text;
using static BlueWest.EfMethods.Sources.Constants;

namespace BlueWest.EfMethods.Sources
{
    internal static class EfGetManyAttributeSource
    {
        internal const string AttributeName = "EfGetMany";
        internal const string AttributeClassName = AttributeName + "Attribute";
        internal const string FullyQualifiedName = RootNamespace + "." + AttributeClassName;

        internal static SourceCode Generate(SourceGenerationOptions options)
        {
            using var builder = new SourceBuilder()
                .WriteLine(GeneratedFilesHeader)
                .WriteLine("using System;")
                .WriteLine()
                .WriteLine($"namespace {RootNamespace}")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Attribute for generating a function to get many entities.")
                .WriteLine("/// </summary>");
            

            builder
                .WriteLine("[AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]")
                .WriteLine($"public sealed class {AttributeName}Attribute : Attribute")
                .WriteOpeningBracket();
            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Gets many records of a entity.")
                .WriteLine("/// <param name=\"returnType\">The projection with the return data.</param>")
                .WriteLine("/// </summary>");
            builder
                .WriteLine($"public {AttributeName}Attribute(Type returnType){"{}"}")
                .WriteLine();

            builder
                .WriteClosingBracket() // class
                .WriteClosingBracket(); // namespace

            return new(builder.ToString(), $"{AttributeName}Attribute.g.cs");
        }
    }
}
