﻿using System;
using System.Collections.Generic;
using System.Text;
using static BlueWest.EfMethods.Sources.Constants;

namespace BlueWest.EfMethods.Sources
{
    internal class EfGetOneByAttributeSource
    {
        internal const string AttributeName = "EfGetOneBy";
        internal const string AttributeClassName = AttributeName + "Attribute";
        internal const string FullyQualifiedName = RootNamespace + "." + AttributeClassName;

        internal static SourceCode Generate(SourceGenerationOptions options)
        {
            using var builder = new SourceBuilder()
                .WriteLine(GeneratedFilesHeader)
                .WriteLine("using System;")
                .WriteLine()
                .WriteLine($"namespace {RootNamespace}")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Generate Get One Entity helper functions with the specified nameof of the property, and the return type.")
                .WriteLine("/// </summary>");
            builder
                .WriteLine("[AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]")
                .WriteLine($"public sealed class {AttributeName}Attribute : Attribute")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Gets one record of a entity by a look up key.")
                .WriteLine("/// <param name=\"bynameof\">The lookup key.</param>")
                .WriteLine("/// <param name=\"returnType\">The projection with the return data.</param>")
                .WriteLine("/// </summary>");
            
            builder
                .WriteLine($"public {AttributeName}Attribute(string bynameof, Type returnType)")
                .WriteOpeningBracket()
                .WriteClosingBracket()
                .WriteLine();

            builder
                .WriteClosingBracket() // class
                .WriteClosingBracket(); // namespace

            return new(builder.ToString(), $"{AttributeName}Attribute.g.cs");
        }
    }
}
