﻿using BlueWest.EfMethods.Sources;
using BlueWest.EfMethods;
using System;
using System.Collections.Generic;
using System.Text;
using static BlueWest.EfMethods.Sources.Constants;

namespace BlueWest.Sources
{
    internal static class EfGetOneFromListAttributeSource
    {
        internal const string AttributeName = "EfGetOneFromList";
        internal const string AttributeClassName = AttributeName + "Attribute";
        internal const string FullyQualifiedName = RootNamespace + "." + AttributeClassName;

        internal static SourceCode Generate(SourceGenerationOptions options)
        {
            using var builder = new SourceBuilder()
                .WriteLine(GeneratedFilesHeader)
                .WriteLine("using System;")
                .WriteLine()
                .WriteLine($"namespace {RootNamespace}")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Attribute for generating a function to get many entities.")
                .WriteLine("/// </summary>");


            builder
                .WriteLine("[AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]")
                .WriteLine($"public sealed class {AttributeName}Attribute : Attribute")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Gets one record of a list in a entity by the specified look up keys.")
                .WriteLine("/// <param name=\"entityPrimaryKeyNameof\">The lookup key of this entity.</param>")
                .WriteLine("/// <param name=\"listMemberNameof\">A nameof with the list in the main type.</param>")
                .WriteLine("/// <param name=\"listPrimaryKeyNameof\">A nameof with the lookup key of the the list type.</param>")
                .WriteLine("/// <param name=\"returnType\">The projection with the returning data.</param>")
                .WriteLine("/// </summary>");

            builder
                .WriteLine($"public {AttributeName}Attribute(string entityPrimaryKeyNameof, string listMemberNameof, string listPrimaryKeyNameof, Type returnType){"{}"}")
                .WriteLine();

            builder
                .WriteClosingBracket() // class
                .WriteClosingBracket(); // namespace

            return new(builder.ToString(), $"{AttributeName}Attribute.g.cs");
        }
    }
}
