using static BlueWest.EfMethods.Sources.Constants;

namespace BlueWest.EfMethods.Sources
{
    public class EfUpdateMethodsAttributeSource
    {
        internal const string AttributeName = "EfUpdateMethods";
        internal const string AttributeClassName = AttributeName + "Attribute";
        internal const string FullyQualifiedName = RootNamespace + "." + AttributeClassName;

        internal static SourceCode Generate(SourceGenerationOptions options)
        {
            using var builder = new SourceBuilder()
                .WriteLine(GeneratedFilesHeader)
                .WriteLine("using System;")
                .WriteLine()
                .WriteLine($"namespace {RootNamespace}")
                .WriteOpeningBracket();

            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Generate update methods this EF entity")
                .WriteLine("/// </summary>");

            builder
                .WriteLine("[AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]")
                .WriteLine($"public sealed class {AttributeName}Attribute : Attribute")
                .WriteOpeningBracket();

            
            builder
                .WriteLine("/// <summary>")
                .WriteLine("/// Updates a entity with projection data.")
                .WriteLine("/// <param name=\"updateType\">A projection type with the update data.</param>")
                .WriteLine("/// <param name=\"returnType\">Type of the projection with the return data.</param>")
                .WriteLine("/// <param name=\"keyPropertyMemberName\">A nameof with the lookup key of this entity.</param>")
                .WriteLine("/// </summary>");

            builder
                .WriteLine($"public {AttributeName}Attribute(Type updateType, Type returnType, string keyPropertyMemberName){"{}"}")
                .WriteLine();

            builder
                .WriteClosingBracket() // class
                .WriteClosingBracket(); // namespace

            return new(builder.ToString(), $"{AttributeName}Attribute.g.cs");
        }
    }
}