﻿using System.Linq;
using BlueWest.EfMethods.Extensions;
using BlueWest.EfMethods.Sources;
using Microsoft.CodeAnalysis;
using Microsoft.CodeAnalysis.CSharp.Syntax;
using static BlueWest.EfMethods.Sources.Constants;

namespace BlueWest.EfMethods
{
    internal static class DiagnosticsFactory
    {
        private const string UsageCategory = "Usage";
        private const string CodePrefix = "MT";
        private const string ErrorId = CodePrefix + "0";
        private const string InfoId = CodePrefix + "1";
        private const string WarningId = CodePrefix + "2";

        internal static Diagnostic TypeNotFoundError(Location location, string syntaxName) =>
            Create($"{ErrorId}010", location, $"Unable to find '{syntaxName}' type.");
        
        internal static Diagnostic NoMatchingPropertyFoundError(Location location, INamedTypeSymbol classType, INamedTypeSymbol sourceType) =>
            Create($"{ErrorId}030", location, $"No matching properties found between '{classType.ToDisplayString()}' and '{sourceType.ToDisplayString()}' types.");

        internal static Diagnostic ConfigurationParseError(string error) =>
            Create($"{ErrorId}040", Location.None, error);

        internal static Diagnostic MissingConstructorArgument(ConstructorDeclarationSyntax constructorSyntax) =>
            Create($"{ErrorId}050", constructorSyntax.GetLocation(), "There are no argument given that corresponds to the required formal parameter.");

        private static Diagnostic Create(string id, Location? location, string message, DiagnosticSeverity severity = DiagnosticSeverity.Error) =>
            Diagnostic.Create(new DiagnosticDescriptor(id, string.Empty, message, UsageCategory, severity, true), location ?? Location.None);
    }
}