﻿using BlueWest.EfMethods.Extensions;
using System.Text;
using BlueWest.EfMethods.Sources;

namespace BlueWest.EfMethods
{
    internal static class EfAddEntityTemplateSource
    {
        internal static SourceBuilder EfAddAddMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfAddMethodsModel model, string addSourceTemplate)
        {
            var returnTypeFullName = model.ReturnTypeFullName;
            var createTypeFullName = model.CreateTypeFullName;
            var newEntityVarName = $"new{model.EntityTypeIdentifierName}";
            var toCreateVarName = $"{model.EntityTypeIdentifierName.ToCamelCase()}ToCreate";
            var entityTypeName = model.EntityTypeIdentifierName;
            var entityFullTypeName = model.EntityTypeFullName;
            var contextFullName = methodsModel.ContextFullType;
            var propertyName = model.PropertyName;

            if (!addSourceTemplate.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(addSourceTemplate);
                templateToSourceBuilder
                    .Replace("{entityTypeName}", model.EntityTypeIdentifierName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{createTypeFullName}", createTypeFullName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{toCreateVarName}", toCreateVarName)
                    .Replace("{newEntityVarName}", newEntityVarName)
                    .Replace("{entityTypeFullName}", model.EntityTypeFullName)
                    .Replace("{propertyName}", propertyName);

                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (addSourceTemplate.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    //.WriteLine(GeneratedFilesHeader)
                    .WriteLine($"public static (bool, {model.ReturnTypeFullName}) Add{entityTypeName}(")
                    .WriteLine($"this {contextFullName} dbContext,")
                    .WriteLine($"{createTypeFullName} {toCreateVarName})")
                    .WriteOpeningBracket()
                    .WriteLine($"var {newEntityVarName} = new {model.EntityTypeFullName}({toCreateVarName});")
                    .WriteLine($"dbContext.{propertyName}.Add({newEntityVarName});")
                    .WriteLine($"var success = dbContext.SaveChanges() >= 0;")
                    .WriteLine($"return (success, new {returnTypeFullName}({newEntityVarName}));")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }
    }
}
