using System.Text;
using BlueWest.EfMethods.Extensions;
using BlueWest.EfMethods.Sources;

namespace BlueWest.EfMethods
{
    internal static class EfAddToListTemplateSource
    {
        internal static SourceBuilder EfAddAddToListMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfAddToListModel model, string template)
        {
            var returnTypeFullName = model.ReturnTypeFullName;
            var entityTypeName = model.EntityTypeIdentifierName;
            var listPropertyName = model.ListPropertyName;
            var listEntityCreateFullName = model.CreateTypeFullName;
            var listItemCreateVarName = model.ListEntityIdentifierName.ToCamelCase() + "toCreate";
            var listEntityTypeName = model.ListEntityIdentifierName;
            var listEntityFullName = model.ListEntityFullTypeName;
            var entityFullTypeName = model.EntityTypeFullName;
            var contextFullName = methodsModel.ContextFullType;
            var propertyName = model.PropertyName;
            var primaryKeyFullName = model.KeyFullTypeName;
            var primaryKeyPropertyName = model.KeyPropertyName;
            var primaryKeyVarName = model.KeyPropertyName.ToCamelCase();
            var entityObjectVarName = model.EntityTypeIdentifierName.ToCamelCase();
    

            if (!template.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(template);
                templateToSourceBuilder
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{entityTypeName}", entityTypeName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{listPropertyName}", listPropertyName)
                    .Replace("{primaryKeyPropertyName}", primaryKeyPropertyName)
                    .Replace("{primaryKeyFullName}", primaryKeyFullName)
                    .Replace("{primaryKeyVarName}", primaryKeyVarName)
                    .Replace("{entityObjectVarName}", entityObjectVarName)
                    .Replace("{propertyName}", propertyName)
                    .Replace("{listEntityCreateFullName}", listEntityCreateFullName)
                    .Replace("{listEntityTypeName}", listEntityTypeName)
                    .Replace("{listEntityFullName}", listEntityFullName)
                    .Replace("{listItemCreateVarName}", listItemCreateVarName)
                    .Replace("{primaryKeyPropertyName}", primaryKeyPropertyName);
                

                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (template.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    //.WriteLine(GeneratedFilesHeader)
                    .WriteLine($"public static (bool, string, {returnTypeFullName}) Add{listEntityTypeName}To{entityTypeName}(  this {contextFullName} dbContext, {primaryKeyFullName} {primaryKeyVarName}, {listEntityCreateFullName} {listItemCreateVarName})")
                    .WriteOpeningBracket()
                    .WriteLine($"var entityQuery = from aEntity in dbContext.{propertyName}")
                    .WriteLine($"where aEntity.{primaryKeyVarName} == {primaryKeyVarName}")
                    .WriteLine($"let itemsInList = aEntity.{listPropertyName}")
                    .WriteLine($"select aEntity;")
                    .WriteLine()
                    .WriteLine($"var entity = entityQuery.FirstOrDefault();")
                    .WriteLine($"if (entity == null) return (false, $\"{entityTypeName} not found.\", null);")
                    .WriteLine($"var newListItem = new {listEntityFullName}({listItemCreateVarName});")
                    .WriteLine($"entity.{listPropertyName}.Add({listItemCreateVarName});")
                    .WriteLine($"var success = dbContext.SaveChanges() >= 0;")
                    .WriteLine($"return !success ? (false, \"Error saving changes in the Database. Action: Create {listEntityTypeName} in {entityTypeName}.\", null) : (true, string.Empty, new {returnTypeFullName}(newListItem));")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }
    }
}