using System.Text;
using BlueWest.EfMethods.Extensions;
using BlueWest.EfMethods.Sources;

namespace BlueWest.EfMethods
{
    public static class EfGetListTemplateSource
    {
        internal static SourceBuilder EfAddGetListMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfGetListModel model, string addSourceTemplate)
        {
            var returnTypeFullName = model.ReturnTypeFullName;
            var entityTypeName = model.EntityTypeIdentifierName;
            var entityFullTypeName = model.EntityTypeFullName;
            var listPropertyName = model.ListPropertyName;
            var contextFullName = methodsModel.ContextFullType;
            var propertyName = model.PropertyName;
            var primaryKeyFullName = model.KeyFullTypeName;
            var primaryKeyPropertyName = model.KeyPropertyName;
            var primaryKeyVarName = model.KeyPropertyName.ToCamelCase();
            var entityObjectVarName = model.EntityTypeIdentifierName.ToCamelCase();

            if (!addSourceTemplate.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(addSourceTemplate);
                templateToSourceBuilder
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{entityTypeName}", entityTypeName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{listPropertyName}", listPropertyName)
                    .Replace("{primaryKeyPropertyName}", primaryKeyPropertyName)
                    .Replace("{primaryKeyFullName}", primaryKeyFullName)
                    .Replace("{primaryKeyVarName}", primaryKeyVarName)
                    .Replace("{entityObjectVarName}", entityObjectVarName)
                    .Replace("{propertyName}", propertyName);

                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (addSourceTemplate.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    //.WriteLine(GeneratedFilesHeader)
                    .WriteLine($"public static (bool, {returnTypeFullName}[]) Get{entityTypeName}{listPropertyName}(this {contextFullName} dbContext, {primaryKeyFullName} {primaryKeyVarName},")
                    .WriteLine($"int skip = 0, int take = 50, int orderDir = 1, Expression<Func<{returnTypeFullName}, bool>> orderBy = null)")
                    .WriteOpeningBracket()
                    .WriteLine($"var {entityObjectVarName} = dbContext.{propertyName}.FirstOrDefault(d => d.{primaryKeyPropertyName} == {primaryKeyVarName});")
                    .WriteLine($"if ({entityObjectVarName} == null) return (false, null);")
                    .WriteLine($"var currentTake = take;")
                    .WriteLine($"if (take > 200) currentTake = 200;")
                    .WriteLine($"var query = dbContext")
                    .WriteLine($".{propertyName}")
                    .WriteLine($".Where(data => data.{primaryKeyPropertyName} == {primaryKeyVarName})")
                    .WriteLine($".SelectMany(o => o.{listPropertyName})")
                    .WriteLine($".Select(x => new {returnTypeFullName}(x))")
                    .WriteLine($".Skip(skip)")
                    .WriteLine($"if(orderBy != null) query = orderDir == 1 ? query.OrderBy(orderBy) : query.OrderByDescending(orderBy);")
                    .WriteLine($"return (query.Any(), query.ToArray());")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }
    }
}