﻿using BlueWest.EfMethods.Extensions;
using BlueWest.EfMethods.Sources;
using System;
using System.Collections.Generic;
using System.Text;

namespace BlueWest.EfMethods
{
    internal static class EfGetOneEntityByTemplate
    {
        internal static SourceBuilder EfAddGetOneEntityByMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfGetOneByModel model, string addSourceTemplate)
        {
            var returnTypeFullName = model.ReturnTypeFullName;
            var entityTypeName = model.EntityTypeIdentifierName;
            var byParamPropertyName = model.ByParamPropertyName;
            var byParamFullType = model.ByParamFullTypeName;

            var contextFullName = methodsModel.ContextFullType;
            var byParamVarName = model.ByParamPropertyName.ToCamelCase();
            var findEntityVarName =  model.EntityTypeIdentifierName.ToCamelCase();
            var propertyName = model.PropertyName;
            var entityFullTypeName = model.EntityTypeFullName;

            if (!addSourceTemplate.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(addSourceTemplate);
                templateToSourceBuilder
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{entityTypeName}", entityTypeName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{byParamPropertyName}", byParamPropertyName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{byParamFullType}", byParamFullType)
                    .Replace("{byParamVarName}", byParamVarName)
                    .Replace("{findEntityVarName}", findEntityVarName)
                    .Replace("{propertyName}", propertyName);

                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (addSourceTemplate.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    //.WriteLine(GeneratedFilesHeader)
                    .WriteLine($"public static (bool, {returnTypeFullName}) GetOne{entityTypeName}By{byParamPropertyName} (this {contextFullName} dbContext, {byParamFullType} {byParamVarName})")
                    .WriteOpeningBracket()
                    .WriteLine($"var {findEntityVarName} = dbContext.{propertyName}")
                    .WriteLine($".Where(x => x.{byParamPropertyName} == {byParamVarName})")
                    .WriteLine($".Select(x => new {returnTypeFullName}(x))")
                    .WriteLine($".FirstOrDefault();")
                    .WriteLine($"return ({findEntityVarName} != null, {findEntityVarName});")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }
    }
}
