﻿using BlueWest.EfMethods.Extensions;
using BlueWest.EfMethods.Sources;
using System;
using System.Collections.Generic;
using System.Text;

namespace BlueWest.EfMethods
{
    internal static class EfGetOneFromListTemplateSource
    {
        internal static SourceBuilder EfAddGetOneFromListMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfGetOneFromListModel model, string addSourceTemplate)
        {
            var returnTypeFullName = model.ReturnTypeFullName;
            var entityTypeName = model.EntityTypeIdentifierName;
            var entityFullTypeName = model.EntityTypeFullName;
            var propertyName = model.PropertyName;
            var listItemTypeName = model.ListEntityIdentifierName;

            var contextFullName = methodsModel.ContextFullType;
            var primaryKeyFullName = model.KeyFullTypeName;
            var primaryKeyVarName = $"{model.EntityTypeIdentifierName}{model.KeyPropertyName}".ToCamelCase();

            var listPrimaryKeyFullName = model.ListKeyFullTypeName;
            var listPrimaryKeyVarName = $"{model.ListEntityIdentifierName}{model.ListKeyPropertyName}".ToCamelCase();
            var listPropertyName = model.ListPropertyName;
            var listPrimaryKeyPropertyName = model.ListKeyPropertyName;

            var primaryKeyPropertyName = model.KeyPropertyName;

            if (!addSourceTemplate.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(addSourceTemplate);
                templateToSourceBuilder
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{entityTypeName}", entityTypeName)
                    .Replace("{listItemTypeName}", listItemTypeName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{primaryKeyPropertyName}", primaryKeyPropertyName)
                    .Replace("{listPrimaryKeyPropertyName}", listPrimaryKeyPropertyName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{primaryKeyFullName}", primaryKeyFullName)
                    .Replace("{primaryKeyVarName}", primaryKeyVarName)
                    .Replace("{listPrimaryKeyFullName}", listPrimaryKeyFullName)
                    .Replace("{listPrimaryKeyVarName}", listPrimaryKeyVarName)
                    .Replace("{listPropertyName}", listPropertyName)

                    .Replace("{propertyName}", propertyName);

                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (addSourceTemplate.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    //.WriteLine(GeneratedFilesHeader)
                    .WriteLine($"public static (bool, {returnTypeFullName}) Get{listItemTypeName}From{entityTypeName} (this {contextFullName} dbContext, {primaryKeyFullName} {primaryKeyVarName},")
                    .WriteLine($"{listPrimaryKeyFullName} {listPrimaryKeyVarName})")
                    .WriteOpeningBracket()
                    .WriteLine($"var query =")
                    .WriteLine($"from mainEntity in dbContext.{propertyName}")
                    .WriteLine($"where mainEntity.{primaryKeyPropertyName} == {primaryKeyVarName}")
                    .WriteLine($"let list = mainEntity.{listPropertyName}")
                    .WriteLine($"from listItem in list")
                    .WriteLine($"where listItem.{listPrimaryKeyPropertyName} == {listPrimaryKeyVarName}")
                    .WriteLine($"select new {returnTypeFullName}(listItem);")
                    .WriteLine()
                    .WriteLine("query = countryQuery.SingleOrDefault();")
                    .WriteLine("return (query != null, query);")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }
    }
}
