﻿using BlueWest.EfMethods.Extensions;
using BlueWest.EfMethods.Sources;
using System;
using System.Collections.Generic;
using System.Text;

namespace BlueWest.EfMethods
{
    internal static class EfGetOneWithTemplateSource
    {
        internal static SourceBuilder EfAddGetOneMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfGetOneWithModel model, string addSourceTemplate)
        {
            var returnTypeFullName = model.ReturnTypeFullName;
            var entityTypeName = model.EntityTypeIdentifierName;

            var contextFullName = methodsModel.ContextFullType;
            var findEntityVarName = model.EntityTypeIdentifierName.ToCamelCase();
            var propertyName = model.PropertyName;
            var entityFullTypeName = model.EntityTypeFullName;
            if (!addSourceTemplate.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(addSourceTemplate);
                templateToSourceBuilder
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{entityTypeName}", entityTypeName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{findEntityVarName}", findEntityVarName)
                    .Replace("{propertyName}", propertyName);

                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (addSourceTemplate.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    //.WriteLine(GeneratedFilesHeader)
                    .WriteLine($"public static (bool, {returnTypeFullName}) GetOne{entityTypeName}With (this {contextFullName} dbContext, Expression<Func<{returnTypeFullName},bool>> with) ")
                    .WriteOpeningBracket()
                    .WriteLine($"var {findEntityVarName} =")
                    .WriteLine($".Select(x => new {returnTypeFullName}(x))")
                    .WriteLine($".FirstOrDefault(with);")
                    .WriteLine()
                    .WriteLine($"return ({findEntityVarName} != null, {findEntityVarName});")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }        
        
        internal static SourceBuilder EfAddGetManyMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfGetManyModel model, string addSourceTemplate)
        {
            var returnTypeFullName = model.ReturnTypeFullName;
            var entityTypeName = model.EntityTypeIdentifierName;
            var entityFullTypeName = model.EntityTypeFullName;
            var contextFullName = methodsModel.ContextFullType;
            var findEntityVarName = model.EntityTypeIdentifierName.ToCamelCase();
            var propertyName = model.PropertyName;

            if (!addSourceTemplate.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(addSourceTemplate);
                templateToSourceBuilder
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{entityTypeName}", entityTypeName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{findEntityVarName}", findEntityVarName)
                    .Replace("{propertyName}", propertyName);

                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (addSourceTemplate.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    //.WriteLine(GeneratedFilesHeader)
                    .WriteLine($"public static (bool, ImmutableArray<{returnTypeFullName}>) Get{propertyName}(this {contextFullName}dbContext, Expression <Func<{returnTypeFullName}, bool> > where = null,")
                    .WriteLine($"Expression <Func<{returnTypeFullName}, object> > orderBy = null, int skip = 0, int take = 50, int orderDir = 1)")
                    .WriteOpeningBracket()
                    .WriteLine($"var query = dbContext")
                    .WriteLine($".{propertyName}")
                    .WriteLine($".Select(x => new {returnTypeFullName}(x));")
                    .WriteComment("limit take by 200 records")
                    .WriteLine("if (take > 200) take = 200;")
                    .WriteLine("query.Skip(skip).Take(take);")
                    .WriteLine("if (where != null) query.Where(where);")
                    .WriteLine("if (orderDir == 1) query.OrderBy(orderBy);")
                    .WriteLine("else query.OrderByDescending(orderBy);")
                    .WriteLine($"return query.ToImmutableArray();")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }
    }
}
