﻿using BlueWest.EfMethods.Extensions;
using System;
using System.Collections.Generic;
using System.Text;

namespace BlueWest.EfMethods.Sources
{
    internal static class EfUpdateEntityTemplateSource
    {
        internal static SourceBuilder EfAddUpdateMethod(this SourceBuilder builder, EfMethodsModel methodsModel, EfUpdateMethodsModel model, string updateSourceTemplate)
        {
            var entityTypeName = model.EntityTypeIdentifierName;
            var contextFullName = methodsModel.ContextFullType;
            var propertyName = model.PropertyName;
            var returnTypeFullName = model.ReturnTypeFullName;
            var updateTypeFullName = model.UpdateTypeFullName;
            var updateVarName = $"{entityTypeName.ToCamelCase()}ToUpdate";
            var keyPropertyName = model.KeyPropertyName;
            var keyTypeFullName = model.KeyFullTypeName;
            var existingVarName = entityTypeName.ToCamelCase();
            var entityFullTypeName = model.EntityTypeFullName;
            var keyVarName = entityTypeName.ToCamelCase() + model.KeyPropertyName;
            if (!updateSourceTemplate.IsEmpty())
            {
                var templateToSourceBuilder = new StringBuilder(updateSourceTemplate);
                templateToSourceBuilder
                    .Replace("{entityTypeName}", entityTypeName)
                    .Replace("{returnTypeFullName}", returnTypeFullName)
                    .Replace("{updateTypeFullName}", updateTypeFullName)
                    .Replace("{updateVarName}", updateVarName)
                    .Replace("{contextFullName}", contextFullName)
                    .Replace("{propertyName}", propertyName)
                    .Replace("{keyTypeFullName}", keyTypeFullName)
                    .Replace("{entityFullTypeName}", entityFullTypeName)
                    .Replace("{keyPropertyName}", keyPropertyName)
                    .Replace("{keyVarName}", keyVarName)
                    .Replace("{existingEntityVarName}", existingVarName);
                builder
                    .ParseTemplate(templateToSourceBuilder.ToString());
            }

            if (updateSourceTemplate.IsEmpty())
            {
                builder
                    .WriteComment("Generated body")
                    .WriteLine($"public static (bool, {returnTypeFullName}) Update{entityTypeName}(")
                    .WriteLine($"this {contextFullName} dbContext,")
                    .WriteLine($"{updateTypeFullName} {updateVarName},")
                    .WriteLine($"{keyTypeFullName} {keyVarName})")
                    .WriteOpeningBracket()
                    .WriteLine($"var {existingVarName} = dbContext.{propertyName}.FirstOrDefault(x => x.{keyPropertyName} == {keyVarName});")
                    .WriteLine($"if ({existingVarName} == null) return (false, null);")
                    .WriteLine($"var success = dbContext.SaveChanges() >= 0;")
                    .WriteLine($"return (success, new {returnTypeFullName}({existingVarName}));")
                    .WriteClosingBracket();

                builder
                    .WriteLine();
            }

            return builder;
        }
    }
}
