﻿using System;
using System.Collections.Immutable;
using BlueWest.EfMethods.Extensions;
using Microsoft.CodeAnalysis;
using Microsoft.CodeAnalysis.CSharp;

namespace BlueWest.EfMethods
{
    internal enum AccessModifier
    {
        Public,
        Internal,
        Private
    }

    internal enum NullStaticAnalysisState
    {
        Default,
        Enabled,
        Disabled
    }

    internal record SourceCode(string Text, string HintName);


    internal record EfMethodsModel(
        SourceGenerationOptions Options,
        string Namespace,
        string ContextTypeName,
        string ContextFullType,
        ImmutableArray<EfEntityDataModel> MethodsModels,
        ImmutableArray<string> Usings
    );

    
    record EfTemplates(
        string AddTemplate,
        string UpdateTemplate,
        string GetOneByTemplate,
        string GetOneTemplate,
        string GetManyTemplate,
        string GetListTemplate,
        string AddToListTemplate,
        string GetOneFromListTemplate
    );
    
    internal class EfEntityDataModel
    {
        public string PropertyName { get; set; }
        public string EntityTypeFullName { get; set; }
        public string EntityTypeIdentifierName { get; set; }

        public EfEntityDataModel(string propertyName, string entityTypeFullName, string entityTypeIdentifierName)
        {
            PropertyName = propertyName;
            EntityTypeFullName = entityTypeFullName;
            EntityTypeIdentifierName = entityTypeIdentifierName;
        }
    }


    internal class EfAddMethodsModel : EfEntityDataModel
    {
        public string CreateTypeFullName { get; set; }
        public string CreateTypeIdentifierName { get; set; }
        public string ReturnTypeFullName { get; set; }
        public string ReturnTypeIdentifierName { get; set; }

        public EfAddMethodsModel(EfEntityDataModel entity, string createTypeFullName,
        string createTypeIdentifierName,
        string returnTypeFullName,
        string returnTypeIdentifierName) : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            CreateTypeFullName = createTypeFullName;
            CreateTypeIdentifierName = createTypeIdentifierName;
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            ReturnTypeFullName = returnTypeFullName;
        }
    }

    internal class EfGetOneByModel : EfEntityDataModel
    {
        public string ByParamPropertyName { get; set; }
        public string ByParamFullTypeName { get; set; }
        public string ByParamTypeName { get; set; }
        public string ReturnTypeIdentifierName { get; set; }
        public string ReturnTypeFullName { get; set; }


        public EfGetOneByModel(EfEntityDataModel entity, string byParamPropertyName,
        string byParamFullTypeName,
        string returnTypeFullName,
        string returnTypeIdentifierName) : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            ByParamPropertyName = byParamPropertyName;
            ByParamFullTypeName = byParamFullTypeName;
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            ReturnTypeFullName = returnTypeFullName;
            ByParamTypeName = returnTypeFullName;
        }
    }
    
    internal class EfGetOneWithModel : EfEntityDataModel
    {
        public string ReturnTypeIdentifierName { get; }
        public string ReturnTypeFullName { get;  }

        public EfGetOneWithModel(EfEntityDataModel entity, string returnTypeFullName,
        string returnTypeIdentifierName) : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            ReturnTypeFullName = returnTypeFullName;
        }
    }
        
    internal class EfGetManyModel : EfEntityDataModel
    {
        public string ReturnTypeIdentifierName { get; set; }
        public string ReturnTypeFullName { get; set; }

        public EfGetManyModel(EfEntityDataModel entity, string returnTypeFullName,
        string returnTypeIdentifierName) : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            ReturnTypeFullName = returnTypeFullName;
        }
    }


    internal class EfUpdateMethodsModel : EfEntityDataModel
    {
        public string UpdateTypeFullName;
        public string UpdateTypeIdentifierName;
        public readonly string ReturnTypeFullName;
        public string ReturnTypeIdentifierName;
        public readonly string KeyPropertyName;
        public readonly string KeyFullTypeName;
        public EfUpdateMethodsModel(EfEntityDataModel entity,
        string updateTypeFullName,
        string updateTypeIdentifierName,
        string returnTypeFullName,
        string returnTypeIdentifierName,
        string keyPropertyName,
        string keyFullTypeName) : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            UpdateTypeFullName = updateTypeFullName;
            UpdateTypeIdentifierName = updateTypeIdentifierName;
            ReturnTypeFullName= returnTypeFullName;
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            KeyPropertyName = keyPropertyName;  
            KeyFullTypeName = keyFullTypeName;
        }
    }
    
    
    internal class EfGetListModel : EfEntityDataModel
    {
        public string ListPropertyName { get; set; }
        
        public string ListObjectFullTypeName { get; set; }
        
        
        public string KeyPropertyName { get; set; }
        public string KeyFullTypeName { get; set; }
        public string ReturnTypeIdentifierName { get; set; }
        public string ReturnTypeFullName { get; set; }


        public EfGetListModel(
            EfEntityDataModel entity, 
            string listPropertyName,
            string listObjectFullTypeName,
            string keyPropertyName,
            string keyFullTypeName, 
            string returnTypeIdentifierName, 
            string returnTypeFullName) 
            : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            ListPropertyName = listPropertyName;
            ListObjectFullTypeName = listObjectFullTypeName;
            KeyPropertyName = keyPropertyName;
            KeyFullTypeName = keyFullTypeName;
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            ReturnTypeFullName = returnTypeFullName;
        }
    }
    
    
    internal class EfAddToListModel : EfEntityDataModel
    {
        public string ListPropertyName { get; set; }
        
        public string ListEntityIdentifierName { get; set; }
        public string ListEntityFullTypeName { get; set; }
        
        public string KeyPropertyName { get; set; }
        public string KeyFullTypeName { get; set; }
                
        public string CreateTypeIdentifierName { get; set; }
        public string CreateTypeFullName { get; set; }

        public string ReturnTypeIdentifierName { get; set; }
        public string ReturnTypeFullName { get; set; }

        


        public EfAddToListModel(
            EfEntityDataModel entity, 
            string listPropertyName,
            string listEntityIdentifierName,
            string listEntityFullTypeName,
            string keyPropertyName,
            string keyFullTypeName, 
            string createTypeIdentifierName,
            string createTypeFullName,
            string returnTypeIdentifierName, 
            string returnTypeFullName) 
            : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            ListPropertyName = listPropertyName;
            ListEntityIdentifierName = listEntityIdentifierName;
            ListEntityFullTypeName = listEntityFullTypeName;
            KeyPropertyName = keyPropertyName;
            KeyFullTypeName = keyFullTypeName;
            CreateTypeIdentifierName = createTypeIdentifierName;
            CreateTypeFullName = createTypeFullName;
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            ReturnTypeFullName = returnTypeFullName;
        }
    }
        
    internal class EfGetOneFromListModel : EfEntityDataModel
    {
        public string ListPropertyName { get; set; }
        
        public string ListEntityIdentifierName { get; set; }
        public string ListEntityFullTypeName { get; set; }
        
        public string KeyPropertyName { get; set; }
        public string KeyFullTypeName { get; set; }

                
        public string ListKeyPropertyName { get; set; }
        public string ListKeyFullTypeName { get; set; }

        public string ListKeyTypeName { get; set; }


        public string ReturnTypeIdentifierName { get; set; }
        public string ReturnTypeFullName { get; set; }

        


        public EfGetOneFromListModel(
            EfEntityDataModel entity, 
            string listPropertyName,
            string listEntityIdentifierName,
            string listEntityFullTypeName,
            string keyPropertyName,
            string keyFullTypeName,
            string listKeyPropertyName,
            string listKeyFullTypeName,
            string listKeyTypeName,
            string returnTypeIdentifierName, 
            string returnTypeFullName) 
            : base(entity.PropertyName, entity.EntityTypeFullName, entity.EntityTypeIdentifierName)
        {
            ListPropertyName = listPropertyName;
            ListEntityIdentifierName = listEntityIdentifierName;
            ListEntityFullTypeName = listEntityFullTypeName;
            KeyPropertyName = keyPropertyName;
            KeyFullTypeName = keyFullTypeName;
            ListKeyPropertyName = listKeyPropertyName;
            ListKeyFullTypeName = listKeyFullTypeName;
            ListKeyTypeName = listKeyTypeName;
            ReturnTypeIdentifierName = returnTypeIdentifierName;
            ReturnTypeFullName = returnTypeFullName;
        }
    }
    
    

    internal record SourceGenerationOptions(
        AccessModifier ConstructorAccessModifier,
        AccessModifier GeneratedMethodsAccessModifier,
        bool GenerateXmlDocument,
        bool SupportNullableReferenceTypes,
        bool SupportNullableStaticAnalysis)
    {
        internal static SourceGenerationOptions From(GeneratorExecutionContext context)
        {
            const string allowNullAttributeName = "System.Diagnostics.CodeAnalysis.AllowNullAttribute";
            var supportNullableStaticAnalysis = context.GetBuildGlobalOption(propertyName: nameof(SupportNullableStaticAnalysis), NullStaticAnalysisState.Default);
            var supportNullableReferenceTypes = context.Compilation.Options.NullableContextOptions is NullableContextOptions.Warnings or NullableContextOptions.Enable;

            return new(
                ConstructorAccessModifier: context.GetBuildGlobalOption(propertyName: nameof(ConstructorAccessModifier), AccessModifier.Public),
                GeneratedMethodsAccessModifier: context.GetBuildGlobalOption(propertyName: nameof(GeneratedMethodsAccessModifier), AccessModifier.Public),
                GenerateXmlDocument: context.GetBuildGlobalOption(propertyName: nameof(GenerateXmlDocument), true),
                SupportNullableReferenceTypes: supportNullableReferenceTypes,
                SupportNullableStaticAnalysis: supportNullableStaticAnalysis switch
                {
                    NullStaticAnalysisState.Enabled => true,
                    NullStaticAnalysisState.Disabled => false,
                    _ => context.Compilation is CSharpCompilation { LanguageVersion: >= LanguageVersion.CSharp8 } cs && cs.TypeByMetadataNameExists(allowNullAttributeName)
                }
            );
        }

        public string NullableReferenceSyntax => SupportNullableReferenceTypes ? "?" : string.Empty;
    }
}